/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xalan" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.ibm.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
#if !defined(FUNCTIONSUBSTRING_HEADER_GUARD_1357924680)
#define FUNCTIONSUBSTRING_HEADER_GUARD_1357924680



// Base header file.  Must be first.
#include <XPath/XPathDefinitions.hpp>



#include <cfloat>



#include <PlatformSupport/DoubleSupport.hpp>
#include <PlatformSupport/DOMStringHelper.hpp>



// Base class header file...
#include <XPath/Function.hpp>



#include <XPath/FunctionRound.hpp>
#include <XPath/XObject.hpp>
#include <XPath/XObjectFactory.hpp>
#include <XPath/XPathExecutionContext.hpp>



/**
 * XPath implementation of "substring" function.
 */
//
// These are all inline, even though
// there are virtual functions, because we expect that they will only be
// needed by the XPath class.
class XALAN_XPATH_EXPORT FunctionSubstring : public Function
{
public:

	// These methods are inherited from Function ...

	virtual XObject*
	execute(
			XPathExecutionContext&			executionContext,
			XalanNode*						context,
			int								/* opPos */,
			const XObjectArgVectorType&		args)
	{
		const XObjectArgVectorType::size_type	theArgCount =
				args.size();

		if (theArgCount < 2 || theArgCount > 3)
		{
			executionContext.error("The substring() function takes two or three arguments!",
								   context);
		}

		const XalanDOMString&	theSourceString = args[0]->str();
		const unsigned int		theSourceStringLength = length(theSourceString);

#if !defined(XALAN_NO_NAMESPACES)
		using std::vector;
#endif

		// This buffer will hold the output characters.
		vector<XalanDOMChar>	theBuffer;

		if (theSourceStringLength > 0)
		{
			// The substring can only be as long as the source string,
			// so reserve enough space now.  Also reserve space for
			// the terminating 0.
			theBuffer.reserve(theSourceStringLength);

			// $$$ ToDo: Add support for NaN.

			// Get the value of the second argument...
			const double	theSecondArgValue =
				FunctionRound::getRoundedValue(args[1]->num());

			// Now, total the second and third arguments.  If
			// the third argument is missing, make the total
			// DBL_MAX.
			const double	theTotal =
					theArgCount == 2 ? DBL_MAX :
									   FunctionRound::getRoundedValue(theSecondArgValue + args[2]->num());

			if (DoubleSupport::isNaN(theSecondArgValue) == false &&
				DoubleSupport::isNaN(theTotal) == false &&
				DoubleSupport::isNegativeInfinity(theTotal) == false)
			{
				// Start with 1, since strings are index from 1 in the XPath spec,
				for (unsigned int i = 1; i <= theSourceStringLength; i++)
				{
					// Is the index greater than or equal to the second argument?
					if (i >= theSecondArgValue)
					{
						// Is it less than the sum of the second and
						// third arguments?
						if (i < theTotal)
						{
							// It is, so include the character.
							theBuffer.push_back(charAt(theSourceString, i - 1));
						}
						else
						{
							// It's not, so stop.
							break;
						}
					}
				}
			}
		}

		return executionContext.getXObjectFactory().createString(XalanDOMString(theBuffer.begin(), theBuffer.size()));
	}

#if defined(XALAN_NO_COVARIANT_RETURN_TYPE)
	virtual Function*
#else
	virtual FunctionSubstring*
#endif
	clone() const
	{
		return new FunctionSubstring(*this);
	}

private:

	// Not implemented...
	FunctionSubstring&
	operator=(const FunctionSubstring&);

	bool
	operator==(const FunctionSubstring&) const;
};



#endif	// FUNCTIONSUBSTRING_HEADER_GUARD_1357924680
